(* configure.ml *)

open Conf

let conf_spec =
  [
    (*findlib_check "threads";*)
    findlib_check "lablgtk2";
    param "debug" ~doc:"Enable debugging" (Bool false);
  ]
;;

let conf = configure conf_spec;;

let split = Util.split;;
let prefix x = List.map (Filename.concat x);;

let dirs_common = (split "util ledit libara common configfile");; (* cli gui *)

let database_common = List.fold_left
  (fun db (x,y) -> CmdLineTools.append db x y)

  CmdLineTools.starting_database

  ([[`compile;`ml],["-thread";"-dtypes"];
    [`compile;`ml],["-pp";"camlp4o"];
    [`compile;`ml;`native],["-inline";"99"];
    [`dep;`ml],["-pp";"camlp4o"]]@
    (if conf#get_bool "debug" then
      [[`compile;`byte],["-g"];
       [`link;`byte],["-g"]]
    else
      [])@
    (List.map (fun l -> ([`compile;`ml],["-I";l])) dirs_common))
;;

let _ =
  Printf.printf "----------- Configuration Summary -----------\n";
  print_string (conf#summarize)

let lib x sl =
  let s = Sources.of_list
    (prefix (x^"/")
      (List.concat
        (List.map (fun x -> [Util.new_ext x ".mli";x]) (split sl))))
  in
  let l = new Product.library ~sources:s ~dest:`nowhere (x^"/"^x) in
  l
;;

let get_sources pl =
  List.fold_left (fun l z ->
    let s = 
      match z with
      | `library(x) -> x#sources
      | `executable(x) -> x#sources
      | _ -> []
    in
    s@l) [] pl
;;

let util =
  lib "util" "publication.ml util.ml";;

let ledit =
  lib "ledit" "cursor.ml ledit.ml";;

let common =
  lib "common" "version.ml help.ml";;

let configfile =
  lib "configfile" "configfile.ml configurator.ml oldconfig.ml configuration.ml";;

let ledit =
  lib "ledit" "cursor.ml ledit.ml";;

let libara =
  lib "libara" "ast.ml syntax.mly lexic.mll slurp.ml debver.ml virtual_strings.ml dpkg.ml ara.ml";;

let libs_of l = Libs.of_list (List.map (fun x -> x^"/"^x) (split l));;

let ara =
  let s = Sources.of_list
    (prefix "cli/" (split "debug.ml opt.ml config.ml wrap.ml pager.ml dump.ml cli.ml"))
  in
  let l = libs_of "util ledit libara common configfile" in
  new Product.executable ~sources:s ~libs:l ~dest:`bin "ara"
;;

let xara =
  let s = Sources.of_list
    (prefix "gui/" (split "debug.ml opt.ml config.ml gui.ml"))
  in
  let l = libs_of "util libara common configfile" in
  new Product.executable ~findlibs:["-thread";"-package threads";"-package lablgtk2.init";"-package unix";"-package str"] ~sources:s ~libs:l ~dest:`bin "xara"
;;

let database_cli =
  List.fold_left
  (fun db (x,y) -> CmdLineTools.append db x y)
  database_common
  [[`compile;`ml],["-I";"cli"];
   [`link;`ml;`byte;`exe],["-thread";"threads.cma";"unix.cma";"str.cma"];
   [`link;`ml;`native;`exe],["-thread";"threads.cmxa";"unix.cmxa";"str.cmxa"]]
;;

let cli =
  let pl = [
    `library util;
    `library ledit;
    `library configfile;
    `library common;
    `library libara;
    `executable ara ]
  in
  let allsrc = get_sources pl in
  let cleaned' = Util.unique (List.map (fun x -> Util.new_ext x "annot") allsrc) in
  let p = Package.create pl in
  { p with
      Package.command_line = database_cli;
      Package.cleaned = cleaned' }
;;

let database_gui =
  List.fold_left
  (fun db (x,y) -> CmdLineTools.append db x y)
  database_common
  [[`compile;`ml],["-I";"gui"];
   [`compile;`ml],["-I";"+lablgtk2"]]
;;

let gui =
  let pl = [
    `library util;
    `library configfile;
    `library common;
    `library libara;
    `executable xara ]
  in
  let allsrc = get_sources pl in
  let cleaned' = Util.unique (List.map (fun x -> Util.new_ext x "annot") allsrc) in
  let p = Package.create pl in
  { p with
      Package.command_line = database_gui;
      Package.cleaned = cleaned' }
;;

let _ =
  conf#set_makefile "Makefile.ara";
  AutoMake.output_makefile ~configuration:(conf :> AutoMake.configuration) cli;
  conf#set_makefile "Makefile.xara";
  AutoMake.output_makefile ~configuration:(conf :> AutoMake.configuration) gui
;;
(* ocamlopt -thread unix.cmxa threads.cmxa str.cmxa -w s lablgtk.cmxa gtkInit.cmx gtkThread.cmx -I +lablgtk2
* util/util.cmxa libara/libara.cmxa common/common.cmxa configfile/configfile.cmxa  -o xara.opt    gui/debug.cmx
* gui/opt.cmx gui/config.cmx gui/gui.cmx *)
